package info.bukova.isspst.ui;

import info.bukova.isspst.data.DataModel;
import info.bukova.isspst.services.Service;
import info.bukova.isspst.services.ValidationException;

import java.util.Map;

import org.springframework.security.access.AccessDeniedException;
import org.zkoss.bind.annotation.BindingParam;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.ExecutionArgParam;
import org.zkoss.bind.annotation.Init;
import org.zkoss.bind.annotation.NotifyChange;
import org.zkoss.zul.Messagebox;
import org.zkoss.zul.Window;

public class FormViewModel<T extends DataModel> {
	
	private T dataBean;
	private Map<String, String> errMessages;
	private Service<T> service;
	private boolean newRec;
	private ServiceConstraint<T> constraint;
	
	@Init
	public void init(@ExecutionArgParam("selected") T selected, @ExecutionArgParam("service") Service<T> service) {
		this.dataBean = selected;
		this.service = service;
		constraint = new ServiceConstraint<T>();
		constraint.setDataBean(selected);
		constraint.setService(service);
		if (selected.getId() == 0 && selected.getCreated() == null) {
			newRec = true;
		} else {
			newRec = false;
		}
	}
	
	public ServiceConstraint<T> getConstriant() {
		return constraint;
	}
	
	public T getDataBean() {
		return dataBean;
	}
	
	@Command
	@NotifyChange("errMessages")
	public void save(@BindingParam("window") Window win) {
		try {
			if (newRec) {
				doAdd();
			} else {
				doSave();
			}
			win.detach();
		} catch (ValidationException e) {
			errMessages = e.getMessages();
			String classErr = errMessages.get("CLASS_ERR");
			
			if (classErr == null) {
				classErr = "";
			}
			
			Messagebox.show("Chyba validace", "Chyba", Messagebox.OK, Messagebox.ERROR);
		} catch (AccessDeniedException e) {
			Messagebox.show("K vykobání této operace nemáte dostatečná oprávnění", "Chyba", Messagebox.OK, Messagebox.ERROR);
		} catch (Exception e) {
			e.printStackTrace();
			Messagebox.show("Chyba při ukládání záznamu", "Chyba", Messagebox.OK, Messagebox.ERROR);
		}
	}

	public Map<String, String> getErrMessages() {
		return errMessages;
	}
	
	protected void setErrMesages(Map<String, String> msgs) {
		this.errMessages = msgs;
	}
	
	protected void doSave() {
		service.update(dataBean);
	}
	
	protected void doAdd() {
		service.add(dataBean);
	}
	
	public boolean isCanSave() {
		return true;
	}

}
