package info.bukova.isspst.ui;

import java.beans.BeanInfo;
import java.beans.IntrospectionException;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.List;

import info.bukova.isspst.reporting.Report;
import info.bukova.isspst.reporting.ReportDefinition;
import info.bukova.isspst.reporting.ReportType;

import org.zkoss.bind.annotation.BindingParam;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.ExecutionArgParam;
import org.zkoss.bind.annotation.Init;
import org.zkoss.bind.annotation.NotifyChange;
import org.zkoss.zk.ui.Executions;
import org.zkoss.zk.ui.select.annotation.WireVariable;
import org.zkoss.zul.Window;

public class ReportDialogVM {
	
	private List<Report> reports;
	private Report selected;
	private List<Object> data;
	@WireVariable
	private ReportDefinition reportDefinition;
	
	@Init
	public void init(@ExecutionArgParam("reports") List<Report> reports, 
			@ExecutionArgParam("data") List<Object> data) {
		this.reports = reports;
		this.data = data;
		
		if (data != null && data.size() > 0 && data.get(0).getClass() != reportDefinition.gatDataClass()) {
			reportDefinition.clear();
		}
	}
	
	public List<Report> getReports() {
		return this.reports;
	}
	
	public ReportDefinition getReportDefinition() {
		return reportDefinition;
	}

	public ListChecks<String> getColumns() {
		if (selected == null || selected.getType() != ReportType.DYNAMIC) {
			return null;
		}
		if (data == null || data.size() == 0) {
			return null;
		}
		
		BeanInfo beanInfo;
		try {
			beanInfo = Introspector.getBeanInfo(data.get(0).getClass());
		} catch (IntrospectionException e) {
			return null;
		}
	    PropertyDescriptor[] pds = beanInfo.getPropertyDescriptors();
	    List<String> properties = new ArrayList<String>();
	    for (PropertyDescriptor pd : pds) {
	    	properties.add(pd.getName());
	    }
		ListChecks<String> columns = new ListChecks<String>(reportDefinition.getFieldsToPrint(), properties);
		
		return columns;
	}

	public Report getSelected() {
		return selected;
	}

	@NotifyChange("columns")
	public void setSelected(Report selected) {
		this.selected = selected;
	}
	
	@Command
	public void print(@BindingParam("window") Window window) {
		reportDefinition.setDataSet(data);
		reportDefinition.setReport(selected);
		
		if (window != null) {
			window.detach();
		}
		
		Window reportWin = (Window) Executions.createComponents("/app/report.zul", null, null);
		reportWin.doModal();
	}

}
